{{

uOLED-96-Prop Cog-Based Graphics Code With uSD Card Access
==========================================================

This code demonstrates how to read a uOLED-96-Prop image file
from a uSD card, and write the image to the OLED display.

The OLED display is controlled via a cog for higher display
speed performance. It should be possible with further optimization
to achieve at least 24 fps video capability.

Note: This code is not optimized in any way. I have achieved
sustained read speeds of 316 kbytes per second. Multiblock
SPI write operation has not yet been implemented. A 96x64 image
requires 12,288 bytes of storage (24 512-byte sectors). A 2GB
uSD card could hold over 174,000 images, or 97 minutes of
30 fps video. 

         
}}

CON

  _clkmode        = xtal1 + pll8x   '64 MHZ system clock (x8 PLL multiplier)
  _xinfreq        = 8_000_000       'uOLED-96-Prop uses 8 MHZ crystal
  
  TwoRows = 96 * 2 * 2 'bytes : (96 pixels/row) * (2 bytes/pixel) * (2 rows)


VAR

  Byte DataBuffer[TwoRows] 
 
OBJ

  OLED    : "uOLED-96-Prop-Cog_V1"
  SDFAT   : "fsrw"
  Delay   : "clock"
  
PUB Start  | r

 OUTA[18]~  'Clear LOW
 DIRA[18]~~ 'Set as OUTPUT

 OUTA[19]~  'Clear LOW
 DIRA[19]~~ 'Set as OUTPUT

 OUTA[20]~  'Clear LOW
 DIRA[20]~~ 'Set as OUTPUT
                   
 DELAY.Init(8_000_000) 
 OLED.Start     'Initialize cog-based OLED graphics display
 SDFAT.Mount    'Initialize FAT16 file system with SD card SPI support
 SDFAT.opendir

 repeat 10

   '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ 
    r := SDFAT.popen(string("ben1.txt"), "r")   'Open Test Image File

    OUTA[18]~~ 'Set HIGH
    DisplayImage   'Display image
    OUTA[18]~  'Clear LOW
 
    SDFAT.pclose   'Close File

    Delay.PauseSec(5)  'Wait 5 seconds before turning off display.

   '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    r := SDFAT.popen(string("movie.txt"), "r")   'Open Test Image File

    OUTA[18]~~ 'Set HIGH
    PlayMovie  
    OUTA[18]~  'Clear LOW
 
    SDFAT.pclose   'Close File

    Delay.PauseSec(2)  

'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 
 OLED.Stop
 
PUB DisplayImage  | StartRow,x
   
  repeat StartRow from 0 to 62 step 2   'loop 32x or total of 12,288 bytes 

     OUTA[19]~~ 'Set HIGH  
     SDFAT.pread(@DataBuffer, TwoRows)
     OUTA[19]~  'Clear LOW    
     'repeat x from 0 to (TwoRows-1)
     '   DataBuffer[x] := $FF

     OUTA[20]~~ 'Set HIGH    
     OLED.WriteData(@DataBuffer, StartRow, 2)  'two 96-pixel rows  (384 bytes)
     OUTA[20]~  'Clear LOW

PUB PlayMovie 

  repeat 118
     DisplayImage
                 