{{  
Author: Greg Porter (c) 2007   All rights reserved   

             Redistribution of unmodified code is permitted. 

 ==============  WARNING - USE AT YOUR OWN RISK  ======================
 Use this code at your own risk! While I have successfully uploaded
 this object for use by the uOLED-96-Prop, I cannot assert that it
 will not harm the device in some unforeseen manner. Please be careful
 not to send inappropriate commands to the graphics controller!          
 ======================================================================
 
This Object is used to control the uOLED-96-PROP display directly from a user
program written in SPIN. The START method must be called first. The STOP method
is designed to terminate the object only after having executed a properly timed
shutdown sequence. STOP should be called as part of a system-wide power down
sequence. The WriteData method is designed to write a specified number of rows
to the OLED display. The code has not, by any means, been optimized.

Demonstration Code:
===================

CON

  _clkmode        = xtal1 + pll8x   '64 MHZ system clock (x8 PLL multiplier)
  _xinfreq        = 8_000_000       'uOLED-96-Prop uses 8 MHZ crystal
  
  TwoRows = 96 * 2 * 2  '(96 pixels/row) * (2 bytes/pixel) * (2 rows)

'===============
VAR

  Long DataBuffer[TwoRows] 

'================
OBJ

  Display : "uOLED-96-Prop-Cog_V1"
  SDfat   : "fsrw"

'================  
PUB Start

 sdfat.mount    'pin assignment removed from .mount method
         
 r := sdfat.popen(string("testimage.txt"), "r")   'preloaded image data

 DisplayImage

 sdfat.pclose 

 repeat
    'forever

'================    
PUB DisplayImage  
   
  repeat row from 0 to 62 step 2   'loop 32x or total of 12,288 bytes 

     SDfat.pread(@DataBuffer, TwoRows)
     Display.WriteData(@DataBuffer, row, 2)  'two 96-pixel rows  (384 bytes)
 
}}

CON

 'DEBUGGING PURPOSES ONLY
  'TestPin18 = 18
  'TestPin19 = 19
  'TestPin20 = 20
  'TestPin21 = 21

'*------------------------------------------------------------------------*
'*  General definitions                                                   *
'*------------------------------------------------------------------------*

  ON       =  1
  OFF      =  0
  INPUT    =  1
  OUTPUT   =  0
  ENABLE   =  1
  DISABLE  =  0
  SET      =  1
  CLEAR    =  0

'*------------------------------------------------------------------------*
'*  OLED Interface (PINS)                                                 *
'*------------------------------------------------------------------------*
  CS_OLED         =  8               ' OLED Chip Select Signal
  RESETPIN_OLED   =  9               ' OLED Reset Signal
  D_C             =  10              ' Data/Command
  WR_OLED         =  11              ' OLED Write Signal
  RD_OLED         =  12              ' OLED Read Signal
  CS_VHI          =  13              ' OLED VCC Enable

'*------------------------------------------------------------------------*
'*  OLED-96 Registers                                                     *
'*------------------------------------------------------------------------*
  SET_COLUMN_ADDRESS    =       $15
  SET_ROW_ADDRESS       =       $75
  CONTRAST_RED          =       $81
  CONTRAST_GREEN        =       $82
  CONTRAST_BLUE         =       $83
  CONTRAST_MASTER       =       $87
  CONTRAST_RED_2ND      =       $8A
  CONTRAST_GREEN_2ND    =       $8B
  CONTRAST_BLUE_2ND     =       $8C
  REMAP_COLOUR_SETTINGS =       $A0
  DISPLAY_START_LINE    =       $A1
  DISPLAY_OFFSET        =       $A2
  DISPLAY_NORMAL        =       $A4
  DISPLAY_ALL_ON        =       $A5
  DISPLAY_ALL_OFF       =       $A6
  DISPLAY_INVERSE       =       $A7
  DUTY_CYCLE            =       $A8
  MASTER_CONFIGURE      =       $AD
  DISPLAY_OFF           =       $AE
  DISPLAY_ON            =       $AF
  POWERSAVE_MODE        =       $B0
  PHASE_PRECHARGE       =       $B1
  CLOCK_FREQUENCY       =       $B3
  SET_GRAYSCALE_LUT     =       $B8
  RESET_GRAYSCALE_LUT   =       $B9
  PRECHARGE_VOLTAGE_RGB =       $BB
  SET_VCOMH             =       $BE
  OLED_NOP              =       $E3
  LOCK_COMMAND          =       $FD

  DRAW_LINE             =       $21
  DRAW_RECTANGLE        =       $22
  COPY_AREA             =       $23
  DIM_WINDOW            =       $24
  CLEAR_WINDOW          =       $25
  FILL_ENABLE_DISABLE   =       $26
  SCROLL_SETUP          =       $27
  STOP_SCROLL           =       $2E
  START_SCROLL          =       $2F

  M_256_COLOURS         =       $32
  M_65K_COLOURS         =       $72


'*-------------------------------------------------------------------------*
'*  Screen Related definitions                                             *
'*-------------------------------------------------------------------------*
  X_RES            =               96
  Y_RES            =               64
  MAX_TEXTROWS     =               25

'*-------------------------------------------------------------------------*
'*  Error Code definitions                                                 *
'*-------------------------------------------------------------------------*
  Error_Command_Unknown   = $FF
  Error_Y_Limit_Exceeded  = $FE
   


VAR

    long cog
    long command, param1, param2, param3, param4   'these five longs are used to communicate with cog

PUB start
           
'   Initialize the graphics card!  
'  
   'OUTA[19]~  'Clear LOW
   'DIRA[19]~~ 'Set as OUTPUT
   
   stop   'FIRST: turn off the OLED display (if ON) in a controlled manner
   
   cog := 1 + cognew(@entry, @command)
    
  'par+0   = @command
  'par+4   = @param1
  'par+8   = @param2
  'par+12  = @param3
  'par+16  = @param4

  'OUTA[19]~~ 'Set HIGH
   command := "I"         'Initialize OLED Display Controller
   repeat while command
  'OUTA[19]~  'Clear LOW
  
   if param1  'TRUE(not-0) : ERROR DETECTED
      abort param1         

   return 0
   
pub stop

   if cog

      command := "X"         'stop display in a controlled manner
      repeat while command
         
      cogstop(cog~ - 1)


pub WriteData(buf_address, row_start, row_count)

  'WRITE 'ROW_COUNT' ROWS TO OLED DISPLAY FROM BUF_ADDRESS STARTING AT ROW 'ROW_START'
  
   param1 := buf_address  'pointer to hub address of first pixel to be written to OLED display
   param2 := row_start    'the first OLED display row to be written (0-63 legal)
   param3 := row_count    'the number of rows (96 pixels x 2-bytes/pixel = 192 bytes per row).

  'OUTA[19]~~ 'Set HIGH
   command := "D"         'image data write to graphics screen
   repeat while command   'SEND COMMAND
  'OUTA[19]~  'Clear LOW
  
   if param1              'a non-zero return value signifies an error condition
      abort param1

   return 0
   
pub WriteCommand(OLED_Command, DataValueCnt, Data_Value1, Data_Value2)

  ' comptr  := @"C"
  ' parptr1 := @SSD1331 command value
  ' parptr2 := @number of parameters    (legal values 1 or 2 )
  ' parptr3 := @parameter value 1
  ' parptr4 := @parameter value 2
                          
   param1 := OLED_Command 'OLED display command
                          'WARNING: The wrong command can be dangerous !!! Exercise caution.
                             
   param2 := DataValueCnt 'Legal values: 1 or 2 (different commands require diff # of arguments)
   param3 := Data_Value1  '1st command value
   param4 := Data_Value2  '2nd command value

   command := "C"         'Send a GRAPHICS command directly to the graphics controller
   repeat while command   'SEND COMMAND

   if param1              'a non-zero return value signifies an error condition
      abort param1

   return 0
   
DAT
        org
   
entry

     'POINTERS TO HUB MEMORY 
     '-----------------------
        mov comptr,par       'comptr = @command
        
        mov parptr1,par       
        add parptr1,#4       'parptr1 = @param1

        mov parptr2,parptr1   
        add parptr2,#4       'parptr2 = @param2

        mov parptr3,parptr2   
        add parptr3,#4       'parptr3 = @param3

        mov parptr4,parptr3   
        add parptr4,#4       'parptr4 = @param4
        
      'par+ 0 = @command 
      'par+ 4 = @param1   
      'par+ 8 = @param2  
      'par+12 = @param3        
      'par+16 = @param4
  
'@@@@@@@@@@@@@@@@@@@@@@@@@@@        
' GRAPHICS CONTROLLER SETUP
'@@@@@@@@@@@@@@@@@@@@@@@@@@@

'===========================================
'======== OUTPUTS: INITIAL STATE  ==========
'===========================================

  '-------------------
  'OUTA[CS_OLED]       := 1  
  'OUTA[RESETPIN_OLED] := 1    '**
  'OUTA[D_C]           := 1
  'OUTA[WR_OLED]       := 1
  'OUTA[RD_OLED]       := 1                        
  'OUTA[CS_VHI]        := 0    'CRITICAL: CS_VHI=LOW : "HIGH" VOLTAGE TO DISPLAY DISABLED !! 
  'OUTA[7..0]          := $00  'data=0 on Pins 7:0 (DIRA LSB)
  '----------------------------------------------------
  
       'OUTA := %00000000_00000000_00011111_00000000   ' Set Output Pin States: 0=Low, 1=High       
        mov outa, outa_pattern                         ' Takes effect after "mov dira" instruction
   
'=====================================
'======== ENABLE OUTPUT PINS =========
'=====================================
  
  '-------------------------------------------------------------
  ' CS_OLED         =  8               ' OLED Chip Select Signal
  ' RESETPIN_OLED   =  9               ' OLED Reset Signal
  ' D_C             =  10              ' Data/Command
  ' WR_OLED         =  11              ' OLED Write Signal
  ' RD_OLED         =  12              ' OLED Read Signal
  ' CS_VHI          =  13              ' OLED VCC Enable
  '--------------------------------------------------------------

       'DIRA := %00000000_00000000_00111111_11111111   ' Set Pin Directions: 0=Input, 1=Output       
        mov dira, dira_pattern

'=====================================
'========  DEFINE PIN MASKS  =========
'=====================================

'--- MASK NAMES -----

'CS_OLED_Mask res 1
'RESET_Mask   res 1
'DC_Mask      res 1
'WR_Mask      res 1
'RD_Mask      res 1
'CS_VHI_Mask  res 1

'---- PIN NAMES -----

'CS_OLED         =  8               ' OLED Chip Select Signal
'RESETPIN_OLED   =  9               ' OLED Reset Signal
'D_C             =  10              ' Data/Command
'WR_OLED         =  11              ' OLED Write Signal
'RD_OLED         =  12              ' OLED Read Signal
'CS_VHI          =  13              ' OLED VCC Enable

        mov CS_OLED_Mask,#1
        shl CS_OLED_Mask,#CS_OLED      'OLED Chip Select Pin#

        mov RESET_Mask,#1
        shl RESET_Mask,#RESETPIN_OLED  'OLED Reset Pin#

        mov DC_Mask,#1
        shl DC_Mask,#D_C               'Data/Command Pin#

        mov WR_Mask,#1
        shl WR_Mask,#WR_OLED           'OLED Write Pin#

        mov RD_Mask,#1
        shl RD_Mask,#RD_OLED           'OLED Read Pin#

        mov CS_VHI_Mask,#1
        shl CS_VHI_Mask,#CS_VHI        'OLED VCC (Hi Voltage) Enable Pin#
 
'========================================================

    '======= DEBUGGING PURPOSES ONLY ========
       
       ' andn outa, TestMask18  'Set level LOW
       ' or   dira, TestMask18  'Define as OUTPUT

       ' andn outa, TestMask19  'Set level HIGH   
       ' or   dira, TestMask19  'Define as OUTPUT

       ' andn outa, TestMask20  'Set level LOW
       ' or   dira, TestMask20  'Define as OUTPUT

       ' andn outa, TestMask21  'Set level LOW
       ' or   dira, TestMask21  'Define as OUTPUT

      'Insert this code to generate a short pulse:
      '-------------------------------------------               
      ' or      outa, TestMask18 'Set TestPin18 HIGH  
      ' andn    outa, TestMask18 'Set TestPin18 LOW   


'=========================================================
'========== RESET SSD1331 GRAPHICS CONTROLLER ============
'=========================================================

' Resets the display registers to their default values. If you call this methode, you must call
' InitOLED method. Equavalent to removing power from the display, without the risk of damaging the
' screen electronics. Use the PowerDown_Seq before removing power from the display.

'When RES# input is pulled low, the chip is initialized with the following status:
'---------------------------------------------------------------------------------
' 1. Display is OFF
' 2. 64 MUX Display Mode
' 3. Display start line is set at display RAM address 0
' 4. Display offset set to 0
' 5. Normal segment and display data column address and row address mapping
'    (SEG0 mapped to address 00H and COM0 mapped to address 00H)
' 6. Column address counter is set at 0
' 7. Master contrast control register is set at 0FH
' 8. Individual contrast control registers of color A, B, and C are set at 80H
' 9. Shift register data clear in serial interface
'10. Normal display mode (Equivalent to A4 command) 
                       
  'OUTA[RESETPIN_OLED] := 0
  'DELAY.PauseMSec(100)
  'OUTA[RESETPIN_OLED] := 1
  
  mov      ReStartTime, Delay_100mS
  add      ReStartTime, cnt
 
  andn     outa, RESET_Mask      'Set Reset Pin LOW
  waitcnt  ReStartTime, 0        'Wait for cnt == ReStartTime
  or       outa, RESET_Mask      'Set Reset Pin HIGH                      

'=======================================
'========== POWER-UP SEQUENCE ==========
'=======================================

' Powers on the screen electronics and displays any screen data previously written to GRAM
' You can continue to write to GRAM (the screen) while the display is powered down and on PowerUp,
' the latest screen data will be displayed.

'Power ON sequence:
'------------------
'1. Power ON VDD, VDDIO.
'2. After VDD, VDDIO become stable,
'     set RES# pin LOW (logic low) for at least 3us (t1) and then HIGH (logic high).
'3. After set RES# pin LOW (logic low),
'     wait for at least 3us (t2). Then Power ON VCC.(1)
'     Note:
'     (1) Since an ESD protection circuit is connected between VDD,VDDIO and VCC,
'         VCC becomes lower than VDD whenever VDD,VDDIO is ON and VCC is OFF as shown
'         in the dotted line of VCC in Figure 19 and Figure 20 .
'4. After VCC become stable,
'     send command AFh for display ON. SEG/COM will be ON after 100ms (tAF).

'  Write_cmd(DISPLAY_OFF)
'  OUTA[CS_VHI] := 1         'Enable OLED VCC
'  DELAY.PauseMSec(100)

  mov      Cmd_Value, #DISPLAY_OFF
  call     #Write_Cmd

  or       outa, CS_VHI_Mask     'Set CS_VHI Pin HIGH                      
   
 '--------- 100 mS delay --------
  mov      ReStartTime, Delay_100mS
  add      ReStartTime, cnt
  waitcnt  ReStartTime, 0        'Wait for cnt == ReStartTime
         
'===================================================================================
 
 'Write_cmd(DISPLAY_NORMAL)                      ' Normal display
  mov      Cmd_Value, #DISPLAY_NORMAL
  call     #Write_Cmd
  
 'Write_cmd(CLOCK_FREQUENCY)                     ' clock & frequency
 'Write_cmd($f0)
  mov      Cmd_Value, #CLOCK_FREQUENCY
  call     #Write_Cmd
  mov      Cmd_Value, #$F0
  call     #Write_Cmd
 
 'Write_cmd(DISPLAY_OFFSET)                      'Set display offset
 'Write_cmd($00)                                 '0 hex start
  mov      Cmd_Value, #DISPLAY_OFFSET
  call     #Write_Cmd
  mov      Cmd_Value, #$00
  call     #Write_Cmd
     
 'Write_cmd(DUTY_CYCLE)                          ' Duty
 'Write_cmd(63)                                  ' 63+1
  mov      Cmd_Value, #DUTY_CYCLE
  call     #Write_Cmd
  mov      Cmd_Value, #63
  call     #Write_Cmd
     
 'Write_cmd(MASTER_CONFIGURE)                    ' Set Master Configuration
 'Write_cmd($8e)
  mov      Cmd_Value, #MASTER_CONFIGURE
  call     #Write_Cmd
  mov      Cmd_Value, #$8e
  call     #Write_Cmd
  
 'Write_cmd(DISPLAY_START_LINE)                  'Set display start line
 'Write_cmd($00)                                 ' 00 hex start
  mov      Cmd_Value, #DISPLAY_START_LINE
  call     #Write_Cmd
  mov      Cmd_Value, #$00
  call     #Write_Cmd
   
 'Write_cmd(REMAP_COLOUR_SETTINGS)               ' Set Re-map Color/Depth
 'Write_cmd(_65K_COLOURS)                        ' 65K 8bit R->G->B
  mov      Cmd_Value, #REMAP_COLOUR_SETTINGS
  call     #Write_Cmd
  mov      Cmd_Value, #M_65K_COLOURS
  call     #Write_Cmd
   
 'Write_cmd(CONTRAST_MASTER)                     ' Set master contrast
 'Write_cmd($0f)
  mov      Cmd_Value, #CONTRAST_MASTER
  call     #Write_Cmd
  mov      Cmd_Value, #$0f
  call     #Write_Cmd
   
 'Write_cmd(CONTRAST_RED)                        ' Set contrast current for A 
 'Write_cmd($ff)
  mov      Cmd_Value, #CONTRAST_RED
  call     #Write_Cmd
  mov      Cmd_Value, #$FF
  call     #Write_Cmd
  
 'Write_cmd(CONTRAST_GREEN)                      ' Set contrast current for B 
 'Write_cmd($ff)
  mov      Cmd_Value, #CONTRAST_GREEN
  call     #Write_Cmd
  mov      Cmd_Value, #$FF
  call     #Write_Cmd
  
 'Write_cmd(CONTRAST_BLUE)                       ' Set contrast current for C 
 'Write_cmd($ff)
  mov      Cmd_Value, #CONTRAST_BLUE
  call     #Write_Cmd
  mov      Cmd_Value, #$FF
  call     #Write_Cmd
  
 'Write_cmd(PRECHARGE_VOLTAGE_RGB)               ' Set pre-charge voltage of color A B C
 'Write_cmd($3E)                                 ' option ?? Write_cmd($1c)
  mov      Cmd_Value, #PRECHARGE_VOLTAGE_RGB
  call     #Write_Cmd
  mov      Cmd_Value, #$3E
  call     #Write_Cmd

 'Write_cmd(SET_VCOMH)                           ' Set VcomH
 'Write_cmd($3E)
  mov      Cmd_Value, #SET_VCOMH
  call     #Write_Cmd
  mov      Cmd_Value, #$3E
  call     #Write_Cmd
   
 'Write_cmd(POWERSAVE_MODE)                      ' Power saving mode
 'Write_cmd($00)
  mov      Cmd_Value, #POWERSAVE_MODE
  call     #Write_Cmd
  mov      Cmd_Value, #$00
  call     #Write_Cmd
   
 'Write_cmd(PHASE_PRECHARGE)                     ' Set pre & dis_charge
 'Write_cmd($11)
  mov      Cmd_Value, #PHASE_PRECHARGE
  call     #Write_Cmd
  mov      Cmd_Value, #$11
  call     #Write_Cmd
   
 'Write_cmd(DISPLAY_ON)                          ' Display on
  mov      Cmd_Value, #DISPLAY_ON
  call     #Write_Cmd
  
'  Set_Full_Screen                               ' Set screen boundaries
'====================
  call #Set_Full_Screen

'  Clear_Screen ?? Optional
'===========================


'@@@@@@@@@@@@@@@@@@@@@@@@@@@
'  COMMAND PROCESSING LOOP
'@@@@@@@@@@@@@@@@@@@@@@@@@@@

        mov Cmd_Error, #0         'zero'ed at startup only ! 
        jmp #CmdDetect
        
'====================================
'       END-OF-COMMAND REPLY
                   
CmdZero wrlong Cmd_Error,parptr1  'zero 'error return' variable in hub memory
                                  'Another parameter # should be used to read back
                                  'data values from the OLED display as this one
                                  'always holds the error-return value. 
        mov acca,#0
        wrlong acca,comptr        'zero 'command' variable in hub memory
                                  'Signals COMMAND completion
                                  
'====================================
'   WAIT IN LOOP FOR A NEW COMMAND

CmdDetect

        mov acca, Delay_100uS
        add acca,cnt
        waitcnt acca,#0             'Pause 100 uS between command reception tests
                                    'Reduces power consumption
                            
        mov starttime,cnt           'Keep resetting command 'starttime'

        rdlong acca,comptr wz       'Has COG received a command ? Z=1 (no command)
   if_z jmp #CmdDetect            
   
'===============================
'       WHICH COMMAND ?

        mov Cmd_Error, #0           'Reset Error flag from previous command (if necessary)
          
        cmp acca,#"D" wz
   if_z jmp #DataWrite              'Send DATA from HUB BUFFER to OLED Display

        cmp acca,#"C" wz
   if_z jmp #CmdWrite               'Send COMMAND(VALUE1,etc) to OLED Display

        cmp acca,#"X" wz
   if_z jmp #PowerDown_Seq          'Stop OLED Display

        cmp acca,#"I" wz
   if_z jmp #CmdZero                'Initialization completed, so clear Command & Error flags,
                                    'then wait for first command ....

                        
        mov Cmd_Error,#Error_Command_Unknown    'Unrecognized command
        jmp #CmdZero 


'========================================================
'      WRITE IMAGE DATA FROM HUB BUFFER TO OLED DISPLAY
'========================================================

DataWrite
        
       'A 384-byte hub buffer would hold 192 2-byte pixel words
       '192 pixels will fit in two rows: 96 pixels/row x 2 = 192 pixels total

       '32 separate buffer reads will acquire all 64 rows of the image.
       
       ' comptr  := "D"
       ' parptr1 := hub address of parameter #1, holds buf_addr start
       ' parptr2 := hub address of parameter #2, holds starting row
       ' parptr3 := hub address of parameter #3, holds number of rows
       
        rdlong  Buf_Addr, parptr1   'get buffer address
        rdlong  RowStart, parptr2   'start row# (0-63 legal)
        rdlong  RowCount, parptr3   '# of rows to fill with pixels (1-63 legal)
                                    'each row contains 96 pixels (192 bytes) !
                
       'Define dimensions of the region in graphics memory to be accessed
        mov xStart, #0
        mov xEnd,   #95
        mov yStart, RowStart
        mov yEnd,   yStart
        add yEnd,   RowCount
        sub yEnd,   #1        'yEnd = yStart + (RowCount-1)
                
'        cmp yEnd, #64 wc   'C=1 if yEnd < 64
'IF_NC   mov Cmd_Error, #Error_Y_Limit_Exceeded
'IF_NC   jmp #CmdZero

       'Set_GRAM_Access (0, 95, RowStart, RowStart + RowCount-1)
        call #Set_GRAM_Access
      
       'Count # of pixels to be written  
        mov Pixel_Cnt, #0
:RowAdd add Pixel_Cnt, #96          '96 pixels/row
        djnz RowCount, #:RowAdd
            
        mov ctr, Pixel_Cnt       

'Loop Performance target:  <5.425 uS/pixel (347 clock cycles, or 86 4-cycle assembly instructions)
'---------------------------------------------------------------------------------------------------
:loop                          
       'Get next 16-bit word from hub memory representing pixel color from hub buffer
        rdword  Data_Value, Buf_Addr

       'Send data to OLED graphics memory
        call #WritePixel    'Currently 18 assembly instructions in length
                            'NOTE: Poorly implemented ... no reason to toggle /CS and D/C
                            'pins high/low with each two bytes written. This is just a first
                            'draft !     
        
        add Buf_Addr, #2    'advance pointer to next WORD (+2 bytes) in hub memory buffer
        djnz ctr, #:loop
'---------------------------------------------------------------------------------
                             
        jmp #CmdZero    'zeroes 'command' variable in hub memory
                        'signals operation completed ...          



'===================================================
'      WRITE A COMMAND TO THE OLED DISPLAY
'====================================================

CmdWrite      'not a routine, returns to #CmdZero address at exit ...

      ' comptr  := @"C"
      ' parptr1 := @SSD1331 command value
      ' parptr2 := @number of parameters    (legal values 1 or 2 (currently))
      ' parptr3 := @parameter value 1
      ' parptr4 := @parameter value 2

       'OUTA[D_C] := 0      
        andn     outa, DC_Mask      'Set Data/Command Pin Low   (Command Mode)

        rdlong Cmd_Value, parptr1
        call #Write_Cmd             'Send SSD1331 Graphics Controller Command
        
        rdlong Cmd_Value, parptr3   'Send Command Parameter 1
        call #Write_Cmd

        rdlong acca, parptr2 
        cmp acca, #2 wc             'C=1 if #paras < 2, C=0 if #paras > 1
        
 IF_NC  rdlong Cmd_Value, parptr4   'Send Command Parameter 2
 IF_NC  call #Write_Cmd

        jmp #CmdZero      'zeroes 'command' variable in hub memory
                          'signals operation completed ...
 
 
'===================================================
'        OLED DISPLAY - POWER DOWN SEQUENCE
'====================================================
       
PowerDown_Seq     'This is NOT a called function, it jumps to "CmdZero" at completion

' Used to power down the screen electronics without disturbing the screen data written to GRAM
' Can be used instead of a screensaver to prolong the life of the display and/or conserve power

' Power OFF sequence:
'--------------------
'1. Send command AEh for display OFF.
'2. Power OFF VCC. (1), (2)
'3. Wait for tOFF. Power OFF VDD, VDDIO. (where Minimum tOFF=0ms, Typical tOFF=100ms)

'(1) Since an ESD protection circuit is connected between VDD,VDDIO and VCC,
'    VCC becomes lower than VDD whenever VDD,VDDIO is ON and VCC is OFF as shown
'    in the dotted line of VCC in Figure 19 and Figure 20 .
'(2) VCC should be kept float when it is OFF.

'  Write_cmd(DISPLAY_OFF)   ' STEP1:  SET DISPLAY OFF
'  OUTA[CS_VHI] := 0        ' STEP2:  Disable OLED VCC
'  DELAY.PauseMSec(100)     ' STEP3:  0-100 mS

  mov      Cmd_Value, #DISPLAY_OFF
  call     #Write_Cmd

  andn     outa, CS_VHI_Mask     'Set CS_VHI Pin LOW                      
   
 '--------- 100 mS delay --------
  mov      ReStartTime, Delay_100mS
  add      ReStartTime, cnt
  waitcnt  ReStartTime, 0        'Wait for cnt == ReStartTime
  
  jmp #CmdZero   'signal completion of operation by zeroing Command flag
  
                        
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
'                                  SUBROUTINES
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

'=========       
WritePixel
'=========

'----------------------------------
'  OUTA[D_C] := 1
'  OUTA[CS_OLED] := 0

'  OUTA[WR_OLED] := 0
'  OUTA[7..0] := R.byte[1]   ' MSB
'  OUTA[WR_OLED] := 1

'  OUTA[WR_OLED] := 0
'  OUTA[7..0] := R.byte[0]   ' LSB
'  OUTA[WR_OLED] := 1

'  OUTA[CS_OLED] := 1
   'OUTA[D_C] := 0
'----------------------------------

  '@@@@@@@@@ SELECT DATA MODE @@@@@@@@@
  
   'OUTA[D_C] := 1      
    or       outa, DC_Mask        'Set Data/Command Pin HIGH  (Data Mode)

  '@@@@@@@@@ CHIP SELECT LOW @@@@@@@@@@
    
   'OUTA[CS_OLED] := 0
    andn     outa, CS_OLED_Mask   'Set CS_OLED Pin LOW
  
  '@@@@@@@@@ 1ST WRITE STROBE - MSB @@@@@@@@@@
  '===========================================
   
   'OUTA[WR_OLED] := 0
    andn     outa, WR_Mask        'Set WR_OLED Pin LOW
    
   'OUTA[7..0] := cmd.byte[0]     'Pins 7:0
   '----------------------------------------
    mov      temp, Data_Value   
    shr      temp, #8             'Get high byte of data_value word

    and      temp, #$FF           'Only 8-bits of interest !
    or       outa, temp           'Set all output pins high whose corresponding bit in temp is high.
    
    or       temp, Mask_FFFFFF00  'CRITICAL: Set bits higher than low byte to all 1s so that the 'AND'
                                  'below does not effect their output state !!
                                  
    and      outa, temp           'Clear all output pins low whose corresponding bit in temp is low.

   'At 64 Mhz, a 4-cycle NOP requires 62.5 nS
   'The data setup requirement is 60 nS. Just to be safe will pad one NOP instruction
   'before rising-edge of /WR line (next instruction).
    nop
    
   'OUTA[WR_OLED] := 1
    or       outa, WR_Mask        'Set WR_OLED Pin HIGH    
      
  '@@@@@@@@@ 2ND WRITE STROBE - LSB @@@@@@@@@@
  '===========================================

   'The /WR line must remain HIGH for at least 60 nS before returning low. Just to be
   'safe, another NOP instruction (62.5 nS) of delay is added prior to /WR falling-edge.
    nop
        
   'OUTA[WR_OLED] := 0
    andn     outa, WR_Mask        'Set WR_OLED Pin LOW
    
   'OUTA[7..0] := cmd.byte[0]     'Pins 7:0
   '----------------------------------------
    mov      temp, Data_Value
    
    and      temp, #$FF           'Only low 8-bits of interest !
    or       outa, temp           'Set all output pins high whose corresponding bit in temp is high.

    or       temp, Mask_FFFFFF00  'CRITICAL: Set bits higher than low byte to all 1s so that the 'AND'
                                  'below does not effect their output state !!
   
    and      outa, temp           'Clear all output pins low whose corresponding bit in temp is low.

   'At 64 Mhz, a 4-cycle NOP requires 62.5 nS
   'The data setup requirement is 60 nS. Just to be safe will pad one NOP instruction
   'before rising-edge of /WR line (next instruction).
    nop
    
   'OUTA[WR_OLED] := 1
    or       outa, WR_Mask        'Set WR_OLED Pin HIGH    
  
  '@@@@@@@@@ CHIP SELECT HIGH @@@@@@@@@@@
       
   'OUTA[CS_OLED] := 1
    or       outa, CS_OLED_Mask   'Set CS_OLED Pin HIGH    

  '@@@@@@@@@@ SELECT COMMAND MODE @@@@@@@@@@
  
   'OUTA[D_C] := 0      
    andn     outa, DC_Mask        'Set Data/Command Pin Low   (Command Mode)
        
WritePixel_RET    ret
                     
        
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
' Shifts out one byte of either register address of data placed on pins 7 through 0

'========
Write_Cmd    'PUB Write_cmd (cmd)
'========

   'OUTA[D_C] := 0
    andn     outa, DC_Mask        'Set Data/Command Pin LOW  (Command Mode)
    
   'OUTA[CS_OLED] := 0
    andn     outa, CS_OLED_Mask   'Set CS_OLED Pin LOW
    
   'OUTA[WR_OLED] := 0
    andn     outa, WR_Mask        'Set WR_OLED Pin LOW
    
   'OUTA[7..0] := cmd.byte[0]     'Pins 7:0
   '----------------------------------------
    and      Cmd_Value, #$FF      'Only low 8-bits of interest !
    or       outa, Cmd_Value      'Set all output pins high whose corresponding bit in Cmd_Value is high.

    or       Cmd_Value, Mask_FFFFFF00  'CRITICAL: Set bits higher than low byte to all 1s so that the 'AND'
                                  'below does not effect their output state !!
   
    and      outa, Cmd_Value      'Clear all output pins low whose corresponding bit in Cmd_Value is low.

   'At 64 Mhz, a 4-cycle NOP requires 62.5 nS
   'The data setup requirement is 60 nS. Just to be safe:  pad one NOP instruction
   'before rising-edge of /WR line (next instruction).
    nop
    
   'OUTA[WR_OLED] := 1
    or       outa, WR_Mask        'Set WR_OLED Pin HIGH    
    
   'OUTA[CS_OLED] := 1
    or       outa, CS_OLED_Mask   'Set CS_OLED Pin HIGH    
    
   'OUTA[D_C] := 1
    or       outa, DC_Mask        'Set Data/Command Pin HIGH  (Data Mode)    
    
Write_Cmd_RET    ret


'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

Set_Full_Screen

    mov xStart, #0
    mov xEnd,   #95
    mov yStart, #0
    mov yEnd,   #63

    call #Set_GRAM_Access
    
Set_Full_Screen_RET    ret

'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

' PUB Set_GRAM_Access(xStart, xEnd, yStart, yEnd)
' Sets the upper-left (xStart, yStart) and the lower-right (xEnd, yEnd) corners
' of the area of Graphic RAM that may be written to

Set_GRAM_Access

   'Write_cmd(SET_COLUMN_ADDRESS) 
   'Write_cmd(xStart)
   'Write_cmd(xEnd)
    mov      Cmd_Value, #SET_COLUMN_ADDRESS
    call     #Write_Cmd

    mov      Cmd_Value, xStart
    call     #Write_Cmd
    mov      Cmd_Value, xEnd
    call     #Write_Cmd
    
   'Write_cmd(SET_ROW_ADDRESS)
   'Write_cmd(yStart)
   'Write_cmd(yEnd)
    mov      Cmd_Value, #SET_ROW_ADDRESS
    call     #Write_Cmd

    mov      Cmd_Value, yStart
    call     #Write_Cmd
    mov      Cmd_Value, yEnd
    call     #Write_Cmd
    
Set_GRAM_Access_RET   ret


'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



'*-------------------------------------------------------------------------*
'*  Color Definitions                                                      *
'*-------------------------------------------------------------------------*
                
MAXCOLOUR  long    $FFFF
BLACK      long    $0000
WHITE      long    $FFFF
RED        long    $F800
GREEN      long    $07E0
BLUE       long    $001F
YELLOW     long    RED | GREEN

'*-------------------------------------------------------------------------*
'*  Cog Register Assignments                                               *
'*-------------------------------------------------------------------------*

'=============================
' Graphics Controller SSD1331
'=============================

' CS#  - Input - This pin is the chip select input connecting to the MCU.

' RES# - Input - This pin is reset signal input.
'                When the pin is low, initialization of the chip is executed.
'                Keep this pin high (i.e. connect to VDDIO) during normal operation.

' D/C# - Input - This pin is Data/Command control pin connecting to the MCU.
'                When the pin is pulled high (i.e. connect to VDDIO), the data
'                at D[15:0] will be interpreted as data.
'                When the pin is pulled low, the data at D[15:0] will be interpreted
'                as command.

' WR#  - Input - This pin is read/write control input pin connecting to the MCU interface.
'                When 8080 interface mode is selected, this pin will be the Write (WR#) input.
'                Data write operation is initiated when this pin is pulled low and the chip is
'                selected.

' RD#  - Input - This pin is MCU interface input.
'                When connecting to an 8080-microprocessor, this pin receives the Read (RD#)
'                signal. Read operation is initiated when this pin is pulled low and the chip is
'                selected.

' CS_OLED         =  8               ' OLED Chip Select Signal (CS#)
' RESETPIN_OLED   =  9               ' OLED Reset Signal (RES#)
' D_C             =  10              ' Data/Command (D=HIGH, C#=LOW)
' WR_OLED         =  11              ' OLED Write Signal (WR#)
' RD_OLED         =  12              ' OLED Read Signal (RD#)
' CS_VHI          =  13              ' OLED VCC Enable
        
DIRA_pattern long  %00000000_00000000_00111111_11111111   'Set Pins Direction: 0=Input, 1=Output       
OUTA_pattern long  %00000000_00000000_00011111_00000000   'Set Output Pins Init State: 0=Low, 1=High       

Delay_100mS  long  6_400_000    '0.1 seconds for 64 Mhz system clock
Delay_100uS  long  6_400        '0.1 millisecons for 64 Mhz system clock

Mask_FFFFFF00 long $FF_FF_FF_00

'#######################################
'Debugging Purposes Only

'TestMask18      long      |<TestPin18
'TestMask19      long      |<TestPin19
'TestMask20      long      |<TestPin20
'TestMask21      long      |<TestPin21
 

acca         res 1

comptr       res 1
parptr1      res 1
parptr2      res 1
parptr3      res 1
parptr4      res 1

ctr          res 1 
starttime    res 1 

'==================
'     PIN MASKS
'==================

' CS_OLED         =  8               ' OLED Chip Select Signal (CS#)
' RESETPIN_OLED   =  9               ' OLED Reset Signal (RES#)
' D_C             =  10              ' Data/Command (D=HIGH, C#=LOW)
' WR_OLED         =  11              ' OLED Write Signal (WR#)
' RD_OLED         =  12              ' OLED Read Signal (RD#)
' CS_VHI          =  13              ' OLED VCC Enable

CS_OLED_Mask res 1
RESET_Mask   res 1
DC_Mask      res 1
WR_Mask      res 1
RD_Mask      res 1
CS_VHI_Mask  res 1

'=====================

RestartTime  res 1
Cmd_Value    res 1
Data_Value   res 1
Buf_Addr     res 1

Pixel_Cnt    res 1

xStart       res 1
xEnd         res 1
xCount       res 1

yStart       res 1
yEnd         res 1
yCount       res 1

Temp         res 1
Cmd_Error    res 1

RowStart     res 1
RowCount     res 1